package aexp.instrumentation;

import android.app.Activity;
import android.app.Instrumentation;
import android.content.Context;
import android.content.Intent;
import android.view.KeyEvent;
import android.os.Bundle;
import android.util.Log;

import java.util.Map;

public class CalculatorInstrumentation extends Instrumentation {
    static final String LOG_TAG = "CalculatorInstrumentation";
    static final String EXPECTED_RESULT = "2.9";

    @Override
    public void onCreate(Bundle arguments) {
        super.onCreate(arguments);

        // When this instrumentation is created, we simply want to start
        // its test code off in a separate thread, which will call back
        // to us in onStart().
        start();
    }

    @Override
    public void onStart() {
        super.onStart();
        Log.d( LOG_TAG,"CalculatorInstrumentation started" );        
        Intent intent = new Intent(Intent.ACTION_MAIN);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        intent.setClass(getTargetContext(), Calculator.class);
        Calculator activity = (Calculator)startActivitySync(intent);
        GetResultText grt = new GetResultText( activity );

        // This is the Activity object that was started, to do with as we want.
        Log.d( LOG_TAG, "Calculator activity started" );


// First field
        sendCharacterSync( KeyEvent.KEYCODE_1 );
        sendCharacterSync( KeyEvent.KEYCODE_PERIOD );
        sendCharacterSync( KeyEvent.KEYCODE_5 );
// move to the second field
        sendCharacterSync( KeyEvent.KEYCODE_DPAD_DOWN );
        sendCharacterSync( KeyEvent.KEYCODE_1 );
        sendCharacterSync( KeyEvent.KEYCODE_PERIOD );
        sendCharacterSync( KeyEvent.KEYCODE_4 );
// move to the '+' button
        sendCharacterSync( KeyEvent.KEYCODE_DPAD_DOWN );
        sendCharacterSync( KeyEvent.KEYCODE_DPAD_CENTER );

// Obtain the result
        runOnMainSync( grt );
        String result = grt.getResultField();
        String message = grt.getMessageField();
        Log.d( LOG_TAG, "result: "+result );
        Log.d( LOG_TAG, "message: "+message );
        boolean testResult = EXPECTED_RESULT.equals( result );
        if( testResult )
            Log.d( LOG_TAG, "Success" );            
        else
            Log.d( LOG_TAG, result+"!="+EXPECTED_RESULT );            
            

        // Wait for the activity to finish all of its processing.
        waitForIdleSync();

        // Retrieve the text we should have written...
        Log.i( LOG_TAG,"CalculatorInstrumentation stopped" );

        Intent resultIntent = new Intent( "TESTRESULT" );
        resultIntent.addCategory( "CALCULATOR_TEST_RESULT" );
        resultIntent.putExtra( "testname","test1" );
        resultIntent.putExtra( "result",result );
        resultIntent.putExtra( "message",message );
        resultIntent.putExtra( "testResult",testResult );
        Context myContext = getContext();
        myContext.sendBroadcast( resultIntent );
        finish(Activity.RESULT_OK, resultIntent.getExtras() );
    }

    class GetResultText implements Runnable {
        Calculator calcInstance;
        String resultField = null;
        String messageField = null;

        public GetResultText( Calculator calcInstance ) {
            this.calcInstance = calcInstance;
        }

        public void run() {
            resultField = calcInstance.getResultField();
            messageField = calcInstance.getMessageField();
        }

        public String getResultField() {
            return resultField;
        }

        public String getMessageField() {
            return messageField;
        }
    }
}

